@@ -8,10 +8,12 @@ from sklearn.utils import check_random_state
 from tslearn.soft_dtw_fast import _soft_dtw, _soft_dtw_grad, _jacobian_product_sq_euc
 from sklearn.metrics.pairwise import euclidean_distances
 
-from tslearn.cydtw import dtw as cydtw, dtw_path as cydtw_path, cdist_dtw as cycdist_dtw
+from tslearn.cydtw import dtw as cydtw, dtw_path as cydtw_path, cdist_dtw as cycdist_dtw, \
+    dtw_subsequence_path as cydtw_subsequence_path
 from tslearn.cydtw import lb_envelope as cylb_envelope
 from tslearn.cydtw import sakoe_chiba_mask as cysakoe_chiba_mask, itakura_mask as cyitakura_mask
-from tslearn.cygak import cdist_gak as cycdist_gak, normalized_gak as cynormalized_gak
+from tslearn.cygak import cdist_gak as cycdist_gak, cdist_normalized_gak as cycdist_normalized_gak, \
+    normalized_gak as cynormalized_gak, gak as cygak
 from tslearn.utils import to_time_series, to_time_series_dataset, ts_size, check_equal_size
 
 __author__ = 'Romain Tavenard romain.tavenard[at]univ-rennes2.fr'
@@ -21,6 +23,9 @@ def dtw_path(s1, s2, global_constraint=None, sakoe_chiba_radius=1):
     """Compute Dynamic Time Warping (DTW) similarity measure between (possibly multidimensional) time series and
     return both the path and the similarity.
 
+    DTW is computed as the Euclidean distance between aligned time series, i.e., if :math:`P` is the alignment path:
+    $$DTW(X, Y) = \\\sqrt{\\\sum_{(i, j) \\\in P} (X_{i} - Y_{j})^2}$$
+
     It is not required that both time series share the same size, but they must be the same dimension.
     DTW was originally presented in [1]_.
 
@@ -79,6 +84,9 @@ def dtw(s1, s2, global_constraint=None, sakoe_chiba_radius=1):
     """Compute Dynamic Time Warping (DTW) similarity measure between (possibly multidimensional) time series and
     return it.
 
+    DTW is computed as the Euclidean distance between aligned time series, i.e., if :math:`P` is the alignment path:
+    $$DTW(X, Y) = \\\sqrt{\\\sum_{(i, j) \\\in P} (X_{i} - Y_{j})^2}$$
+
     It is not required that both time series share the same size, but they must be the same dimension.
     DTW was originally presented in [1]_.
 
@@ -126,6 +134,48 @@ def dtw(s1, s2, global_constraint=None, sakoe_chiba_radius=1):
     return cydtw(s1, s2, mask=numpy.zeros((sz1, sz2)))
 
 
+def dtw_subsequence_path(subseq, longseq):
+    """Compute sub-sequence Dynamic Time Warping (DTW) similarity measure between a (possibly multidimensional)
+    query and a long time series and return both the path and the similarity.
+
+    DTW is computed as the Euclidean distance between aligned time series, i.e., if :math:`P` is the alignment path:
+    $$DTW(X, Y) = \\\sqrt{\\\sum_{(i, j) \\\in P} (X_{i} - Y_{j})^2}$$
+
+    It is not required that both time series share the same size, but they must be the same dimension.
+    This implementation finds the best matching starting and ending positions for `subseq` inside `longseq`.
+
+    Parameters
+    ----------
+    subseq
+        A query time series.
+    longseq
+        A reference (supposed to be longer than `subseq`) time series.
+
+    Returns
+    -------
+    list of integer pairs
+        Matching path represented as a list of index pairs. In each pair, the first index corresponds to `subseq` and
+        the second one corresponds to `longseq`
+    float
+        Similarity score
+
+    Examples
+    --------
+    >>> path, dist = dtw_subsequence_path([2, 3], [1., 2., 2., 3., 4.])
+    >>> path
+    [(0, 2), (1, 3)]
+    >>> dist
+    0.0
+
+    See Also
+    --------
+    dtw : Get the similarity score for DTW
+    """
+    subseq = to_time_series(subseq)
+    longseq = to_time_series(longseq)
+    return cydtw_subsequence_path(subseq=subseq, longseq=longseq)
+
+
 def sakoe_chiba_mask(sz1, sz2, radius=1):
     """
     Examples
@@ -165,6 +215,9 @@ def itakura_mask(sz1, sz2):
 def cdist_dtw(dataset1, dataset2=None, global_constraint=None, sakoe_chiba_radius=1):
     """Compute cross-similarity matrix using Dynamic Time Warping (DTW) similarity measure.
 
+    DTW is computed as the Euclidean distance between aligned time series, i.e., if :math:`P` is the alignment path:
+    $$DTW(X, Y) = \\\sqrt{\\\sum_{(i, j) \\\in P} (X_{i} - Y_{j})^2}$$
+
     DTW was originally presented in [1]_.
 
     Parameters
@@ -223,6 +276,7 @@ def gak(s1, s2, sigma=1.):
 
     It is not required that both time series share the same size, but they must be the same dimension. GAK was
     originally presented in [1]_.
+    This is a normalized version that ensures that $k(x,x)=1$ for all $x$ and $k(x,y) \in [0, 1]$ for all $x, y$.
 
     Parameters
     ----------
@@ -255,8 +309,7 @@ def gak(s1, s2, sigma=1.):
     """
     s1 = to_time_series(s1)
     s2 = to_time_series(s2)
-    gamma = 1. / (2 * sigma ** 2)
-    return numpy.exp(-soft_dtw(s1, s2, gamma=gamma) / gamma)  #cynormalized_gak(s1, s2, sigma)
+    return cynormalized_gak(s1, s2, sigma)
 
 
 def cdist_gak(dataset1, dataset2=None, sigma=1.):
@@ -302,7 +355,7 @@ def cdist_gak(dataset1, dataset2=None, sigma=1.):
         self_similarity = True
     else:
         dataset2 = to_time_series_dataset(dataset2)
-    return numpy.array([[gak(s1, s2, sigma=sigma) for s2 in dataset2] for s1 in dataset1])# cycdist_gak(dataset1, dataset2, sigma, self_similarity=self_similarity)
+    return cycdist_normalized_gak(dataset1, dataset2, sigma, self_similarity=self_similarity)
 
 
 def sigma_gak(dataset, n_samples=100, random_state=None):
@@ -538,7 +591,7 @@ def soft_dtw(ts1, ts2, gamma=1.):
     """
     if gamma == 0.:
         return dtw(ts1, ts2)
-    return SoftDTW(SquaredEuclidean(ts1[:ts_size(ts1)], ts2[:ts_size(ts2)]), gamma=numpy.float64(gamma)).compute()
+    return SoftDTW(SquaredEuclidean(ts1[:ts_size(ts1)], ts2[:ts_size(ts2)]), gamma=gamma).compute()
 
 
 def cdist_soft_dtw(dataset1, dataset2=None, gamma=1.):
@@ -572,6 +625,8 @@ def cdist_soft_dtw(dataset1, dataset2=None, gamma=1.):
     See Also
     --------
     soft_dtw : Compute Soft-DTW
+    cdist_soft_dtw_normalized : Cross similarity matrix between time series
+    datasets using a normalized version of Soft-DTW
 
     References
     ----------
@@ -605,6 +660,53 @@ def cdist_soft_dtw(dataset1, dataset2=None, gamma=1.):
     return dists
 
 
+def cdist_soft_dtw_normalized(dataset1, dataset2=None, gamma=1.):
+    """Compute cross-similarity matrix using a normalized version of the
+    Soft-DTW metric.
+
+    Soft-DTW was originally presented in [1]_.
+    This normalized version is defined as:
+    `sdtw_(x,y) := sdtw(x,y) - 1/2(sdtw(x,x)+sdtw(y,y))`
+    and ensures that all returned values are positive and that
+    `sdtw_(x,x) == 0`.
+
+    Parameters
+    ----------
+    dataset1
+        A dataset of time series
+    dataset2
+        Another dataset of time series
+    gamma : float (default 1.)
+        Gamma paraneter for Soft-DTW
+
+    Returns
+    -------
+    numpy.ndarray
+        Cross-similarity matrix
+
+    Examples
+    --------
+    >>> time_series = numpy.random.randn(10, 15, 1)
+    >>> numpy.alltrue(cdist_soft_dtw_normalized(time_series) >= 0.)
+    True
+
+    See Also
+    --------
+    soft_dtw : Compute Soft-DTW
+    cdist_soft_dtw : Cross similarity matrix between time series
+    datasets using the unnormalized version of Soft-DTW
+
+    References
+    ----------
+    .. [1] M. Cuturi, M. Blondel "Soft-DTW: a Differentiable Loss Function for
+       Time-Series," ICML 2017.
+    """
+    dists = cdist_soft_dtw(dataset1, dataset2=dataset2, gamma=gamma)
+    d_ii = numpy.diag(dists)
+    dists -= .5 * (d_ii.reshape((-1, 1)) + d_ii.reshape((1, -1)))
+    return dists
+
+
 class SoftDTW(object):
     def __init__(self, D, gamma=1.):
         """
