@@ -62,6 +62,10 @@ class PiecewiseAggregateApproximation(TransformerMixin):
     n_segments : int
         Number of PAA segments to compute
 
+    Note
+    ----
+        This method requires a dataset of equal-sized time series.
+
     Example
     -------
     >>> paa = PiecewiseAggregateApproximation(n_segments=3)
@@ -255,6 +259,10 @@ class SymbolicAggregateApproximation(PiecewiseAggregateApproximation):
     breakpoints_avg_ : numpy.ndarray of shape (alphabet_size - 1, )
         List of breakpoints used to generate SAX symbols
 
+    Note
+    ----
+        This method requires a dataset of equal-sized time series.
+
     Example
     -------
     >>> sax = SymbolicAggregateApproximation(n_segments=3, alphabet_size_avg=2)
@@ -418,7 +426,7 @@ class SymbolicAggregateApproximation(PiecewiseAggregateApproximation):
         numpy.ndarray of shape (n_ts, sz_original_ts, d)
             A dataset of time series corresponding to the provided representation.
         """
-        X_ = to_time_series_dataset(X, dtype=numpy.int)
+        X_ = numpy.array(X, dtype=numpy.int)
         return inv_transform_sax(X_, breakpoints_middle_=self.breakpoints_avg_middle_, original_size=self.size_fitted_)
 
 
@@ -437,7 +445,7 @@ class OneD_SymbolicAggregateApproximation(SymbolicAggregateApproximation):
         Number of SAX symbols to use to describe slopes.
     sigma_l : float or None (default: None)
         Scale parameter of the Gaussian distribution used to quantize slopes. If None, the formula given in [1]_ is
-        used: :math:`\\sigma_L = \\sqrt{0.03 / L}` where :math:`L` is the length of the considered time series.
+        used: :math:`\\sigma_L = \\sqrt{0.03 / L}` where :math:`L` is the length of each segment.
 
     Attributes
     ----------
@@ -446,6 +454,10 @@ class OneD_SymbolicAggregateApproximation(SymbolicAggregateApproximation):
     breakpoints_slope_ : numpy.ndarray of shape (alphabet_size_slope - 1, )
         List of breakpoints used to generate SAX symbols for slopes.
 
+    Note
+    ----
+        This method requires a dataset of equal-sized time series.
+
     Example
     -------
     >>> one_d_sax = OneD_SymbolicAggregateApproximation(n_segments=3, alphabet_size_avg=2, alphabet_size_slope=2, sigma_l=1.)
@@ -486,7 +498,7 @@ class OneD_SymbolicAggregateApproximation(SymbolicAggregateApproximation):
     Traceback (most recent call last):
     ValueError: Model not fitted yet: cannot be used for distance computation.
     >>> unfitted_1dsax.fit(data).sigma_l  # doctest: +ELLIPSIS
-    0.07071...
+    0.12247...
 
 
     References
@@ -504,8 +516,11 @@ class OneD_SymbolicAggregateApproximation(SymbolicAggregateApproximation):
 
     def _fit(self, X, y=None):
         SymbolicAggregateApproximation._fit(self, X, y)
+
+        n_ts, sz, d = X.shape
+        sz_segment = sz // self.n_segments
         if self.sigma_l is None:
-            self.sigma_l = numpy.sqrt(0.03 / self.size_fitted_)
+            self.sigma_l = numpy.sqrt(0.03 / sz_segment)
 
         self.breakpoints_slope_ = _breakpoints(self.alphabet_size_slope, scale=self.sigma_l)
         self.breakpoints_slope_middle_ = _bin_medians(self.alphabet_size_slope, scale=self.sigma_l)
@@ -654,7 +669,7 @@ class OneD_SymbolicAggregateApproximation(SymbolicAggregateApproximation):
         numpy.ndarray of shape (n_ts, sz_original_ts, d)
             A dataset of time series corresponding to the provided representation.
         """
-        X_ = to_time_series_dataset(X, dtype=numpy.int)
+        X_ = numpy.array(X, dtype=numpy.int)
         return inv_transform_1d_sax(X_,
                                     breakpoints_avg_middle_=self.breakpoints_avg_middle_,
                                     breakpoints_slope_middle_=self.breakpoints_slope_middle_,
